/* 
    qasn1 - nodetest - unit tests for Asn1BerNode class of qasn1 library
    Copyright(C) 2010 - Andrius da Costa Ribas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtCore/QDebug>
#include <QtCore/QObject>
#include <QtTest/QtTest>

#include "Asn1BerNode.h"

#include "nodetest.h"

void Asn1BerNodeTest::initTestCase()
{
    qDebug() << "startup";
}

void Asn1BerNodeTest::testTagClass()
{
    Asn1BerNode node;
    node.setClass(CLASS_APPLICATION);
    
    QVERIFY(node.tagClass() == CLASS_APPLICATION);
    QVERIFY(node.isClassApplication());
}

void Asn1BerNodeTest::testLinks()
{
    Asn1BerNode parentNode;
    Asn1BerNode childNode1(&parentNode);
    Asn1BerNode childNode2;
    Asn1BerNode childNode3;
    Asn1BerNode* nodePointer = new Asn1BerNode;
    
    childNode1.insertBefore(&childNode2); // child2 becomes the first child
    childNode1.insertAfter(&childNode3);
    
    QVERIFY(childNode1.prev() == &childNode2);
    QVERIFY(childNode3.prev() == &childNode1);
    QVERIFY(childNode2.next()->next() == &childNode3);
    QVERIFY(childNode1.parent() == &parentNode);
    QVERIFY(childNode2.parent() == &parentNode);
    QVERIFY(childNode3.parent() == &parentNode);
    QVERIFY(parentNode.parent() == 0);
    QVERIFY(parentNode.firstChild() == &childNode2);
    
    childNode1.insertAfter(nodePointer);
    
    QVERIFY(nodePointer->prev() == &childNode1);
    QVERIFY(childNode1.next() == nodePointer);
    
    delete nodePointer;
    
    QVERIFY(childNode3.parent() == &parentNode);
    QVERIFY(childNode2.next()->next() == &childNode3);
    
}

void Asn1BerNodeTest::testNewNodeState()
{
    Asn1BerNode node;
    
    QVERIFY(node.val().isNull());
    QVERIFY(!(node.hasDefinedLength()));
    QVERIFY(node.isCodingPrimitive());
}

void Asn1BerNodeTest::cleanupTestCase()
{
    qDebug() << "cleanup";
}

// this thing defines a standard main function
QTEST_MAIN(Asn1BerNodeTest)

#include "nodetest.moc"